package com.stmatthews.hospital.facilities;



import com.bea.wlevs.ede.api.RunnableBean;
import com.bea.wlevs.ede.api.StreamSender;
import com.bea.wlevs.ede.api.StreamSource;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import java.util.List;

public class TemperatureSensorSimulator implements RunnableBean, StreamSource {

    private static final int SLEEP_MILLIS = 2500;
    private StreamSender eventSender;
    private boolean suspended;
    private List<TemperatureSensor> sensors = new ArrayList<TemperatureSensor>();
    private Map<String,Rogue> rogues = new HashMap<String,Rogue>();
    private int burstCounter =0;

    public TemperatureSensorSimulator(){
    sensors.add(new TemperatureSensor("XT.0191", "MW.A.100", 23.5f, 1.2f));
    sensors.add(new TemperatureSensor("XT.3921", "MW.A.100", 23.6f, 0.7f));
    sensors.add(new TemperatureSensor("XT.78311", "MW.A.100", 23.4f, 0.3f));

    sensors.add(new TemperatureSensor("XT.4191", "MW.B.100", 23.7f, 0.2f));
    sensors.add(new TemperatureSensor("XT.7921", "MW.B.100", 23.2f, 0.3f));
    sensors.add(new TemperatureSensor("XT.28311", "MW.B.100", 23.6f, 0.1f));

    sensors.add(new TemperatureSensor("AB.8712", "BS.B.005", 4f, 0.9f));
    sensors.add(new TemperatureSensor("AB.958", "BS.B.005", 4f, 0.4f));
    sensors.add(new TemperatureSensor("AB.2351", "BS.B.005", 4f, 0.2f));

    sensors.add(new TemperatureSensor("AB.712", "BS.B.105", 4.3f, 0.3f));
    sensors.add(new TemperatureSensor("AB.7958", "BS.B.105", 4.1f, 0.1f));
    sensors.add(new TemperatureSensor("AB.6351", "BS.B.105", 3.5f, 0.5f));

    sensors.add(new TemperatureSensor("NU.0191", "RM.C.160", 20.5f, 1.3f));
    sensors.add(new TemperatureSensor("NU.0291", "RM.C.160", 20.4f, 0.4f));
    sensors.add(new TemperatureSensor("NU.0792", "RM.C.160", 20.2f, 0.2f));

    sensors.add(new TemperatureSensor("CO.LD321", "RF.X.5", -12f, 1.2f));
    sensors.add(new TemperatureSensor("CO.LD329", "RF.X.5", -13f, 0.5f));
    sensors.add(new TemperatureSensor("CO.LD324", "RF.X.5", -11f, 0.2f));
    }
    
    public void run() {
        suspended = false;
        while (!isSuspended()) { // Generate messages forever...

            generateTemperatureSensorBurst();

            try {
                synchronized (this) {
                    wait(SLEEP_MILLIS);
                }
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }

    private void generateTemperatureSensorBurst() {    	     
    	// update rogues
    	for(String key: rogues.keySet()) {
    		Rogue rogue = rogues.get(key);
    		if (Math.abs(rogue.getCummulativeDelta())< rogue.getMaxDelta()){
    			rogue.setCummulativeDelta((float)(rogue.getCummulativeDelta() + rogue.getDeltaPerBurst()*(0.5 + 1.1*Math.random())));
    		}
    	}
    	
        // send the messages - one for each sensor
        for (TemperatureSensor sensor : sensors) {
          if (!sensor.isBroken()) {
          float temperature =
            sensor.getBaseTemperature() + (float)(Math.random() *                                                  
            		sensor.getRange()) -
            (float)0.5 * sensor.getRange();
          Rogue rogue = (rogues.get(sensor.getClusterId()));
          if (rogue!=null) {
        	  temperature = temperature + rogue.getCummulativeDelta();
          }
          TemperatureReadingEvent event = new TemperatureReadingEvent();          
          event.setTemperature(temperature);
          event.setSensorId(sensor.getSensorId());
          event.setClusterId(sensor.getClusterId());
          eventSender.sendInsertEvent(event);

          }
        } // all sensors simulated once
      //  System.out.println("sent out sensor readings burst "+burstCounter++ );

        // stick a needle in the haystack
        // break a sensor if needs be, once every 50 bursts       
        if ((++burstCounter%50) ==0) {
          int sensorToBreak = (int)(sensors.size()  * Math.random());
          if (!sensors.get(sensorToBreak).isBroken()) {
            System.out.println("Sensor "+sensors.get(sensorToBreak).getSensorId()+" has just broken down (in burst "+burstCounter+")!");
            sensors.get(sensorToBreak).setBroken(true);
          }
        }

        // with a small chance, create a rogue - a cluster on the run
        // an open window/door or a fridge door left open
        // or a fire!
        if (Math.random()<0.02){
        	int sensorId = (int)Math.floor(Math.random()*sensors.size());
        	if (rogues.get(sensors.get(sensorId).getClusterId())==null) { // do not overwrite an existing rogue!
        	  boolean isFire = Math.random()> 0.8;
        	  boolean goUp = sensors.get(sensorId).getBaseTemperature()<10; // test for cooling units
        	  Rogue rogue = new Rogue((float)(isFire?2:(0.1*(goUp?1:-1))),0,isFire?750:8,isFire?"Fire":"Door left open", sensors.get(sensorId).getClusterId());
        	  rogues.put(rogue.getClusterId(), rogue);
          	  System.out.println("Rogue gone wild for cluster "+rogue.getClusterId()+" - "+rogue.getRogueType()+" delta per burst "+rogue.getDeltaPerBurst() );
        	}
        	
        }
	}

	/* (non-Javadoc)
     * @see com.bea.wlevs.ede.api.SuspendableBean#suspend()
     */
    public synchronized void suspend() {
        suspended = true;
    }

    private synchronized boolean isSuspended() {
        return suspended;
    }
    public void setEventSender(StreamSender sender) {
        eventSender = sender;
    }
    
}
